;;; Sierra Script 1.0 - (do not remove this comment)
;;;;
;;;;  SAVE.SC
;;;;
;;;;  (c) Sierra On-Line, Inc, 1992
;;;;
;;;;  Author:  Jeff Stephenson
;;;;  Updated: Brian K. Hughes
;;;;
;;;;  Classes which create the save/restore game user interface.  Also
;;;;  contains a number of instances of Dialogs and associated DItems
;;;;  used in the interface.
;;;;
;;;;  Classes:
;;;;     SRDialog
;;;;     Save
;;;;     Restore

(script# SAVE) ;SAVE = 990
(include system.sh) (include sci2.sh) (include game.sh)
(use Main)
(use Interface)
(use System)

(define  GAMESSHOWN 8)     ;the number of games displayed in the selector
(define  MAXGAMES 20)      ;maximum number of games in a save directory
(define  COMMENTSIZE 36)   ;size of user's description of the game
(define  COMMENTBUFF 18) 	;(/ (+ 1 COMMENTSIZE) 2))

(define  DIRECTORYSIZE 29) ;size of the save directory name
(define  DIRECTORYBUFF 15)	;(/ (+ 1 DIRECTORYSIZE) 2))

(define  BUFFERSIZE 361)	;(+ (* MAXGAMES COMMENTBUFF) 1))

(public
	GetDirectory 0
)

(local
	default
	i
	numGames
	selected
	theStatus
	[butbuf1 4] = [{Restore} {__Save__} {Replace} {Replace}]
	[butbuf2 4] = [{Select the game that you would like to restore.} {Type the description of this saved game.} {This directory/disk can hold no more saved games. You must replace one of your saved games or use Change Directory to save on a different directory/disk.} {This directory/disk can hold no more saved games. You must replace one of your saved games or use Change Directory to save on a different directory/disk.}]
)

(enum
   RESTORE        ;Restore games
   HAVESPACE      ;Save, with space on disk
   NOSPACE        ;Save, no space on disk but games to replace
   NOREPLACE      ;Save, no space on disk, no games to replace
)


(procedure (GetDirectory where &tmp result [newDir 33] [str 40])
	;(procedure (GetDirectory where &tmp result 
    ;        [newDir 40] [str 100] [buf1 50] len
    ;        )

	(asm
code_0748:
		pushi    13
		pushi    990
		pushi    1
		pushi    33
		pushi    0
		pushi    41
		pushi    2
		lea      @newDir
		push    
		lsp      where
		callk    StrCpy,  4
		push    
		pushi    29
		pushi    81
		lofsa    {OK}
		push    
		pushi    1
		pushi    81
		lofsa    {Cancel}
		push    
		pushi    0
		calle    Print,  26
		sat      result
		not     
		bnt      code_077b
		ldi      0
		ret     
code_077b:
		pushi    1
		lea      @newDir
		push    
		callk    StrLen,  2
		not     
		bnt      code_078f
		pushi    1
		lea      @newDir
		push    
		callk    GetCWD,  2
code_078f:
		pushi    1
		lea      @newDir
		push    
		callk    ValidPath,  2
		bnt      code_07aa
		pushi    2
		lsp      where
		lea      @newDir
		push    
		callk    StrCpy,  4
		ldi      1
		ret     
		jmp      code_0748
code_07aa:
		pushi    3
		pushi    4
		lea      @str
		push    
		pushi    990
		pushi    2
		lea      @newDir
		push    
		callk    Format,  8
		push    
		pushi    33
		pushi    0
		calle    Print,  6
		jmp      code_0748
		ret     
	)
)

(procedure (GetStatus)
	(return
		(cond 
			((== self Restore) 	RESTORE)
			((HaveSpace) 		HAVESPACE)
			(numGames 			NOSPACE)
			(else 				NOREPLACE)
		)
	)
)

(procedure (HaveSpace)
	(if (< numGames MAXGAMES) (CheckFreeSpace curSaveDir))
)

(procedure (NeedDescription)
	(Print SAVE 3 #font 0)
	; You must type a description for the game.
)

(class SysWindow of Object
	;; The window class defines visible BORDERED rectangular areas of the screen.
	;; A window, ID'ed by systemWindow, is used by Dialog to specify various
	;; aspects of its appearence. 

	(properties
		top 		0		
		left 		0		
		bottom 		0		
		right 		0		
		color 		vBLACK	; foreground color
		back 		vWHITE	; background color
		priority 	-1		; priority
		window 		0		; handle/pointer to system window
		type 		$0000	; generally corresponds to system window types
		title 		0		; text appearing in title bar if present

		;; this rectangle is the working area for X/Y centering
		;; these coordinates can define a subsection of the picture
		;; in which a window will be centered
		brTop 		0
		brLeft 		0
		brBottom 	190
		brRight 	SCRNWIDE
	)
	
	(method (dispose)
		(if window (DisposeWindow window) (= window NULL))
		(super dispose:)
	)
	
	(method (open)
		;; Open corresponding system window structure
		;; Custom window type 0x81 indicates that system
		;; will NOT draw the window, only get a port and link into list
		(= window
			(NewWindow
				;; port dimensions
				top
				left
				bottom
				right
				
				title
				type
				priority
				color
				back
			)
		)
	)
)

(class SRDialog of Dialog
	;;; The SRDialog class implements the user interface for save/restore.
	;;; Its subclasses are the specific save and restore game dialogs,
	;;; Save and Restore.

	(properties
		elements 	0
		size 		0
		text 		0
		window 		0
		theItem 	0
		nsTop 		0
		nsLeft 		0
		nsBottom 	0
		nsRight 	0
		time 		0
		busy 		0
		seconds 	0
		lastSeconds 0
	)
	
	(method (init theComment names nums)
		;; Initialize the dialog.
		
		; give ourself the system window as our window
		(= window SysWindow)
		
		;Re-init our size, with no elements.
		(= nsBottom 0)
		
		;Get some files for this directory.
		(if
			(==
				(= numGames (GetSaveFiles (theGame name?) names nums))
				-1
			)
			(return FALSE)
		)
		
		;Set up the edit item for saved games.
		(if (== (= theStatus (GetStatus)) HAVESPACE)
			(editI
				text: (StrCpy theComment names)
				font: smallFont
				setSize:
				moveTo: MARGIN MARGIN
			)
			(self add: editI setSize:)
		)
		
		;Set up the selectorI box.
		(selectorI
			text: names
			font: smallFont
			setSize:
			moveTo: MARGIN (+ nsBottom MARGIN)
			state: dExit
		)
		(= i (+ (selectorI nsRight?) MARGIN))
		(okI
			text: [butbuf1 theStatus]
			setSize:
			moveTo: i (selectorI nsTop?)
			state: (if (== theStatus NOREPLACE) 0 else (| dActive dExit))
		)
		(cancelI
			setSize:
			moveTo: i (+ (okI nsBottom?) MARGIN)
			state: (& (cancelI state?) (~ dSelected))
		)
		(changeDirI
			setSize:
			moveTo: i (+ (cancelI nsBottom?) MARGIN)
			state: (& (changeDirI state?) (~ dSelected))
		)
		(self add: selectorI okI cancelI changeDirI setSize:)
		(textI
			text: [butbuf2 theStatus]
			setSize: (- (- nsRight nsLeft) (* 2 MARGIN))
			moveTo: MARGIN MARGIN
		)
		(= i (+ (textI nsBottom?) MARGIN))
		(self eachElementDo: #move 0 i)
		(self add: textI setSize: center: open: wTitled pWHITE)
		(return TRUE)
	)
	
	(method (doit theComment &tmp fd retrn offset temp3 [names 361] [nums 21] [dir 40])
		(asm
			pushSelf
			lofsa    Restore
			eq?     
			bnt      code_026d
			lap      argc
			bnt      code_026d
			lap      theComment
			bnt      code_026d
			pushi    1
			pushi    4
			lea      @dir
			push    
			pushi    990
			pushi    0
			pushi    #name
			pushi    0
			lag      theGame
			send     4
			push    
			callk    Format,  8
			push    
			callk    FOpen,  2
			sat      retrn
			push    
			ldi      65535
			eq?     
			bnt      code_0267
			ret     
code_0267:
			pushi    1
			lst      retrn
			callk    FClose,  2
code_026d:
			pushi    #init
			pushi    3
			lsp      theComment
			lea      @names
			push    
			lea      @nums
			push    
			self     10
			not     
			bnt      code_0286
			ldi      65535
			ret     
code_0286:
			lsl      theStatus
			dup     
			ldi      0
			eq?     
			bnt      code_02a0
			lal      numGames
			bnt      code_02bd
			lofsa    okI
			jmp      code_02bd
			lofsa    changeDirI
			jmp      code_02bd
code_02a0:
			dup     
			ldi      1
			eq?     
			bnt      code_02ad
			lofsa    editI
			jmp      code_02bd
code_02ad:
			dup     
			ldi      2
			eq?     
			bnt      code_02ba
			lofsa    okI
			jmp      code_02bd
code_02ba:
			lofsa    changeDirI
code_02bd:
			toss    
			sal      default
			pushi    #doit
			pushi    1
			push    
			super    Dialog,  6
			sal      i
			pushi    #indexOf
			pushi    1
			pushi    #cursor
			pushi    0
			lofsa    selectorI
			send     4
			push    
			lofsa    selectorI
			send     6
			sal      selected
			push    
			ldi      18
			mul     
			sat      temp3
			lsl      i
			lofsa    changeDirI
			eq?     
			bnt      code_0384
			pushi    1
			lsg      curSaveDir
			call     GetDirectory,  2
			bnt      code_0286
			pushi    3
			pushi    #name
			pushi    0
			lag      theGame
			send     4
			push    
			lea      @names
			push    
			lea      @nums
			push    
			callk    GetSaveFiles,  6
			sal      numGames
			push    
			ldi      65535
			eq?     
			bnt      code_031c
			ldi      65535
			sat      offset
			jmp      code_0494
code_031c:
			lal      theStatus
			sat      fd
			pushi    0
			call     GetStatus,  0
			sal      theStatus
			push    
			dup     
			ldi      0
			eq?     
			bnt      code_0332
			jmp      code_0378
code_0332:
			dup     
			lat      fd
			eq?     
			bnt      code_0363
			pushi    #contains
			pushi    1
			lofsa    editI
			push    
			self     6
			bnt      code_0378
			pushi    #cursor
			pushi    1
			pushi    1
			pushi    2
			lsp      theComment
			lea      @names
			push    
			callk    StrCpy,  4
			push    
			callk    StrLen,  2
			push    
			pushi    83
			pushi    0
			lofsa    editI
			send     10
			jmp      code_0378
code_0363:
			pushi    #dispose
			pushi    0
			pushi    87
			pushi    3
			lsp      theComment
			lea      @names
			push    
			lea      @nums
			push    
			self     14
code_0378:
			toss    
			pushi    #draw
			pushi    0
			lofsa    selectorI
			send     4
			jmp      code_0286
code_0384:
			lsl      theStatus
			ldi      2
			eq?     
			bnt      code_03ba
			lsl      i
			lofsa    okI
			eq?     
			bnt      code_03ba
			pushi    #doit
			pushi    1
			pushi    2
			lsp      theComment
			lat      temp3
			leai     @names
			push    
			callk    StrCpy,  4
			push    
			lofsa    GetReplaceName
			send     6
			bnt      code_0286
			lal      selected
			lati     nums
			sat      offset
			jmp      code_0494
			jmp      code_0286
code_03ba:
			lsl      theStatus
			ldi      1
			eq?     
			bnt      code_043b
			lsl      i
			lofsa    okI
			eq?     
			bt       code_03d4
			lsl      i
			lofsa    editI
			eq?     
			bnt      code_043b
code_03d4:
			pushi    1
			lsp      theComment
			callk    StrLen,  2
			push    
			ldi      0
			eq?     
			bnt      code_03e9
			pushi    0
			call     NeedDescription,  0
			jmp      code_0286
code_03e9:
			ldi      65535
			sat      offset
			ldi      0
			sal      i
code_03f1:
			lsl      i
			lal      numGames
			lt?     
			bnt      code_0413
			pushi    2
			lsp      theComment
			lsl      i
			ldi      18
			mul     
			leai     @names
			push    
			callk    StrCmp,  4
			sat      offset
			not     
			bnt      code_040e
code_040e:
			+al      i
			jmp      code_03f1
code_0413:
			lat      offset
			not     
			bnt      code_0421
			lal      i
			lati     nums
			jmp      code_0433
code_0421:
			lsl      numGames
			ldi      20
			eq?     
			bnt      code_0431
			lal      selected
			lati     nums
			jmp      code_0433
code_0431:
			lal      numGames
code_0433:
			sat      offset
			jmp      code_0494
			jmp      code_0286
code_043b:
			lsl      i
			lofsa    okI
			eq?     
			bnt      code_0451
			lal      selected
			lati     nums
			sat      offset
			jmp      code_0494
			jmp      code_0286
code_0451:
			lsl      i
			ldi      0
			eq?     
			bt       code_0462
			lsl      i
			lofsa    cancelI
			eq?     
			bnt      code_046c
code_0462:
			ldi      65535
			sat      offset
			jmp      code_0494
			jmp      code_0286
code_046c:
			lsl      theStatus
			ldi      1
			eq?     
			bnt      code_0286
			pushi    #cursor
			pushi    1
			pushi    1
			pushi    2
			lsp      theComment
			lat      temp3
			leai     @names
			push    
			callk    StrCpy,  4
			push    
			callk    StrLen,  2
			push    
			pushi    83
			pushi    0
			lofsa    editI
			send     10
			jmp      code_0286
code_0494:
			pushi    #dispose
			pushi    0
			self     4
			lat      offset
			ret     
		)
	)
)

(class Restore of SRDialog
	(properties
		elements 	0
		size 		0
		text 		{Restore a Game}
		window 		0
		theItem 	0
		nsTop 		0
		nsLeft 		0
		nsBottom 	0
		nsRight 	0
		time 		0
		busy 		0
		seconds 	0
		lastSeconds 0
	)
)

(class Save of SRDialog
	(properties
		elements 	0
		size 		0
		text 		{Save a Game}
		window 		0
		theItem 	0
		nsTop 		0
		nsLeft 		0
		nsBottom 	0
		nsRight 	0
		time 		0
		busy 		0
		seconds 	0
		lastSeconds 0
	)
)

(instance GetReplaceName of Dialog
	(properties)
	
	(method (doit theComment &tmp ret)
		
		; give ourself the system window as our window
		(= window SysWindow)
		(text1 setSize: moveTo: MARGIN MARGIN)
		(self add: text1 setSize:)
		(oldName
			text: theComment
			font: smallFont
			setSize:
			moveTo: MARGIN nsBottom
		)
		(self add: oldName setSize:)
		(text2 setSize: moveTo: MARGIN nsBottom)
		(self add: text2 setSize:)
		(newName
			text: theComment
			font: smallFont
			setSize:
			moveTo: MARGIN nsBottom
		)
		(self add: newName setSize:)
		(button1 nsLeft: 0 nsTop: 0 setSize:)
		(button2 nsLeft: 0 nsTop: 0 setSize:)
		(button2
			moveTo: (- nsRight (+ (button2 nsRight?) MARGIN)) nsBottom
		)
		(button1
			moveTo: (- (button2 nsLeft?) (+ (button1 nsRight?) MARGIN)) nsBottom
		)
		(self add: button1 button2 setSize: center: open: stdWindow pWHITE)
		(= ret (super doit: newName))
		(self dispose:)
		(if (not (StrLen theComment))
			(NeedDescription)
			(= ret 0)
		)
		(return (if (== ret newName) else (== ret button1)))
	)
)

(instance selectorI of DSelector
	(properties
		x COMMENTSIZE
		y GAMESSHOWN 
	)
)

(instance editI of DEdit
	(properties
		max (- COMMENTSIZE 1)
	)
)

(instance okI of DButton
	(properties)
)

(instance cancelI of DButton
	(properties
		text { Cancel_}
	)
)

(instance changeDirI of DButton
	(properties
		text {Change\nDirectory}
	)
)

(instance textI of DText
	(properties
		font SYSFONT
	)
)

(instance text1 of DText
	(properties
		text {Replace}
		font SYSFONT
	)
)

(instance text2 of DText
	(properties
		text {with:}
		font SYSFONT
	)
)

(instance oldName of DText
	(properties)
)

(instance newName of DEdit
	(properties
		max (- COMMENTSIZE 1)
	)
)

(instance button1 of DButton
	(properties
		text {Replace}
	)
)

(instance button2 of DButton
	(properties
		text {Cancel}
	)
)
